%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Hazard curves plot 
%
% Author: Demi Girot (dgirot@usgs.gov)
%         Peter Powers (pmpowers@usgs.gov)
%
% Updated: 05/23/2023
%
% This script demonstrates how to call web services for the hazard 
% calculations implemented in USGS National Sesmic Hazard Models and plot 
% the results. A JSON web service response is automatically converted to a
% struct by the Matlab webread() function. This script shows how to plot
% the source location (lat, lon) requested.
%
% Users are STRONGLY encouraged to run the web services locally so as to
% not overburden USGS servers. See the repository README for details on how
% to do so: https://code.usgs.gov/ghsc/nshmp/nshmp-haz
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Plot a figure of hazard curves

clearvars

% The root url for the hazard curves web service:
urlbase = "https://earthquake.usgs.gov/ws/nshmp/conus-2018/dynamic/hazard/";

% If you are running nshmp-haz locally, use this URL:
% urlbase = "http://localhost:8080/hazard/";

% Location input for plot title  
at.location = 'Denver, Colorado';

% Struct of input parameters:
in.longitude = -104.992;
in.latitude = 39.74;
in.vs30 = 760;
in.truncate = false;
in.maxdir = false;
in.imt = 'SA0P2';

figure
plot_handles = [];

hazard = readHazard(urlbase, in).response.hazardCurves;
plotHazard(hazard, in, at)

%% Shared Functions

function plotHazard(hazard, in, at)

    plot_handles = [];
    legend_labels = {};

    % Struct for hazard data
    data = hazard.data;

    % imt display in plot title
    imt = hazard.imt.display;

    for j = 1 : length(data)

        hazard_xs = data(j).values.xs;
        hazard_ys = data(j).values.ys;
    
        % Plot the total spectrum
        ph = loglog(hazard_xs, hazard_ys, ...
            'LineWidth',3, ...
            'LineStyle','-');
        
        color = get(ph,'Color');
        hold on;
        
        plot_handles = [plot_handles ph];
        legend_labels{end+1} = data(j).component;
    end

    grid on

    xlabel('Ground Motion (g)')
    ylabel('Annual Frequency of Exceedance')
    title(string(at.location),'FontSize',20)
    subtitle(['(' num2str(in.latitude) ',' num2str(in.longitude) '), '...
        num2str(imt)],'FontSize',16)

    set(gca,'FontSize',20)

    legend(plot_handles,legend_labels,'Location','best');

end

% Read hazard curves from a NSHM web service for the supplied source 
% location (lat,lon) and input parameters
function response = readHazard(urlbase, in)

url = urlbase;

url = url + ...
    num2str(in.longitude) + ...
    "/" + num2str(in.latitude) + ...
    "/" + num2str(in.vs30) + ...
    "?truncate=" + string(in.truncate) + ...
    "&maxdir=" + string(in.maxdir) + ...
    "&imt=" + string(in.imt);

% Timeout increased to compute hazard calculations
options = weboptions('Timeout',30);
response = webread(url,options);

end