package gov.usgs.earthquake.nshmp.www.source;

import gov.usgs.earthquake.nshmp.www.NshmpMicronautServlet;
import gov.usgs.earthquake.nshmp.www.ResponseBody;
import gov.usgs.earthquake.nshmp.www.ServletUtil;
import gov.usgs.earthquake.nshmp.www.source.FeaturesService.FeatureType;
import gov.usgs.earthquake.nshmp.www.source.FeaturesService.Metadata;
import gov.usgs.earthquake.nshmp.www.source.FeaturesService.RequestData;

import io.micronaut.core.annotation.Nullable;
import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.micronaut.http.annotation.PathVariable;
import io.micronaut.http.annotation.QueryValue;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.inject.Inject;

/**
 * Micronaut web service controller for the GeoJSON features in the installed
 * model.
 *
 * <p>See src/main/resources/application.yml nshmp-haz.model-path for installed
 * model.
 *
 * <p>To run the Micronaut jar file with a model: java -jar
 * path/to/nshmp-haz.jar --model=<path/to/model>
 *
 * @author U.S. Geological Survey
 */
@Tag(
    name = FeaturesService.NAME,
    description = "USGS NSHM source model features service")
@Controller("${nshmp-haz.context-path}/features")
public class FeaturesController {

  @Inject
  private NshmpMicronautServlet servlet;

  @Operation(
      summary = "Source model features listing",
      description = "Returns the features in a source model",
      operationId = "source-features")
  @ApiResponse(
      description = "NSHM source features",
      responseCode = "200",
      content = @Content(
          schema = @Schema(
              implementation = MetadataResponse.class)))
  @Get(uri = "{?featureType,raw}")
  public HttpResponse<String> doGet(
      HttpRequest<?> http, @QueryValue @Nullable FeatureType featureType,
      @QueryValue(defaultValue = "false") @Nullable Boolean raw) {
    try {
      return FeaturesService.getFeatures(http, featureType, raw);
    } catch (Exception e) {
      return ServletUtil.error(
          FeaturesService.LOG, e,
          FeaturesService.NAME,
          http.getUri().toString());
    }
  }

  @Operation(
      summary = "Get the GeoJSON features in a source model",
      description = "Returns the features for the supplied feature type",
      operationId = "source-features-slash")
  @ApiResponse(
      description = "NSHM source features",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = TreeResponse.class)))
  @Get(uri = "/{featureType}")
  public HttpResponse<String> doGetSlash(
      HttpRequest<?> http,
      @PathVariable FeatureType featureType) {
    return doGet(http, featureType, null);
  }

  @Operation(
      summary = "Get the GeoJSON features in a source model",
      description = "Returns the features for the supplied feature type",
      operationId = "source-features-slash-raw")
  @ApiResponse(
      description = "NSHM source features",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = TreeResponse.class)))
  @Get(uri = "/{featureType}/{raw}")
  public HttpResponse<String> doGetSlashRaw(
      HttpRequest<?> http,
      @PathVariable FeatureType featureType,
      @PathVariable Boolean raw) {
    return doGet(http, featureType, raw);
  }

  // Swagger schema
  private static class MetadataResponse extends ResponseBody<String, Metadata> {}

  // Swagger schema
  private static class TreeResponse extends ResponseBody<RequestData, Object> {}
}
