package gov.usgs.earthquake.nshmp.www.source;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.CaseFormat;

import gov.usgs.earthquake.nshmp.model.HazardModel;
import gov.usgs.earthquake.nshmp.model.Models;
import gov.usgs.earthquake.nshmp.model.SourceType;
import gov.usgs.earthquake.nshmp.www.HazVersion;
import gov.usgs.earthquake.nshmp.www.ResponseBody;
import gov.usgs.earthquake.nshmp.www.ResponseMetadata;
import gov.usgs.earthquake.nshmp.www.ServletUtil;
import gov.usgs.earthquake.nshmp.www.meta.Parameter;
import gov.usgs.earthquake.nshmp.www.source.SourceService.SourceModel;

import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.swagger.v3.oas.models.media.Schema;
import io.swagger.v3.parser.util.SchemaTypeUtil;
import jakarta.inject.Singleton;

/**
 * @author U.S. Geological Survey
 */
@Singleton
public class FeaturesService {

  static final String NAME = "Model Features";
  static final Logger LOG = LoggerFactory.getLogger(FeaturesService.class);

  public static HttpResponse<String> getFeatures(
      HttpRequest<?> request,
      FeatureType featureType,
      Boolean raw) {
    if (featureType == null) {
      return metadata(request);
    } else {
      return processResponse(request, new RequestData(featureType, raw));
    }
  }

  /**
   * Updates the {@code FeatureType} enum schema to Swagger.
   *
   * @param schemas The Swagger schemas
   */
  @SuppressWarnings("rawtypes")
  public static void featureTypeSchema(Map<String, Schema> schemas) {
    var schema = new Schema<FeatureType>();
    schema.setType(SchemaTypeUtil.STRING_TYPE);
    featureTypes().forEach(featureType -> schema.addEnumItemObject(featureType));

    schemas.put(
        String.format(
            "%s.%s",
            FeaturesService.class.getSimpleName(),
            FeatureType.class.getSimpleName()),
        schema);
  }

  private static Set<FeatureType> featureTypes() {
    Set<FeatureType> featureTypes = new TreeSet<>();
    featureTypes.add(FeatureType.ALL);

    Arrays.stream(FeatureType.values())
        .sorted()
        .filter(featureType -> featureType != FeatureType.ALL)
        .forEach(featureType -> {
          var features = Models.features(
              ServletUtil.model(),
              Optional.of(SourceType.valueOf(featureType.name())));

          if (features.isPresent()) {
            featureTypes.add(featureType);
          }
        });

    return featureTypes;
  }

  private static HttpResponse<String> processResponse(
      HttpRequest<?> request,
      RequestData requestData) {
    var url = request.getUri().toString();

    var features = requestData.featureType == FeatureType.ALL
        ? Models.features(ServletUtil.model(), Optional.empty())
        : Models.features(
            ServletUtil.model(),
            Optional.of(SourceType.valueOf(requestData.featureType.name())));

    if (features.isEmpty()) {
      throw new IllegalStateException(
          String.format("No features found for type %s", requestData.featureType));
    }

    if (requestData.raw) {
      return HttpResponse.ok(features.orElseThrow().toJson());
    } else {
      var response = ResponseBody.success()
          .name(NAME)
          .url(url)
          .metadata(new ResponseMetadata(HazVersion.appVersions(ServletUtil.model().root())))
          .request(requestData)
          .response(features.orElseThrow().toJsonTree())
          .build();

      return HttpResponse.ok(ServletUtil.GSON2.toJson(response));
    }
  }

  private static HttpResponse<String> metadata(HttpRequest<?> request) {
    var url = request.getUri().toString();
    var metadata = new Metadata(ServletUtil.model());
    var response = ResponseBody.usage()
        .name(NAME)
        .url(url)
        .metadata(new ResponseMetadata(HazVersion.appVersions(ServletUtil.model().root())))
        .request(url)
        .response(metadata)
        .build();
    return HttpResponse.ok(ServletUtil.GSON2.toJson(response));
  }

  static class RequestData {
    final FeatureType featureType;
    final boolean raw;

    RequestData(FeatureType featureType, Boolean raw) {
      this.raw = raw == null ? false : raw;
      this.featureType = featureType;
    }

    public FeatureType getFeatureType() {
      return featureType;
    }

    public boolean getRaw() {
      return raw;
    }
  }

  /**
   * Subset of {@code SourceType}
   */
  static enum FeatureType {
    ALL,
    DECOLLEMENT,
    FAULT,
    INTERFACE,
    ZONE;

    @Override
    public String toString() {
      return CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, name());
    }
  }

  static class Metadata {
    final SourceModel model;
    final List<Parameter> featureType;

    Metadata(HazardModel model) {
      this.model = new SourceModel(model);
      this.featureType = featureTypes().stream()
          .map(featureType -> new Parameter(featureType.toString(), featureType.name()))
          .collect(Collectors.toList());
    }

    public List<Parameter> getFeatureType() {
      return featureType;
    }

    public SourceModel getModel() {
      return model;
    }
  }
}
