package gov.usgs.earthquake.nshmp.www.source;

import gov.usgs.earthquake.nshmp.www.NshmpMicronautServlet;
import gov.usgs.earthquake.nshmp.www.ResponseBody;
import gov.usgs.earthquake.nshmp.www.ServletUtil;
import gov.usgs.earthquake.nshmp.www.source.LogicTreesService.Metadata;
import gov.usgs.earthquake.nshmp.www.source.LogicTreesService.RequestData;

import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.micronaut.http.annotation.PathVariable;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.inject.Inject;

/**
 * Micronaut web service controller for the source logic trees in the installed
 * model. These endpoints return the GeoJSON features and Magnitude frequency
 * distributions associated with the branches in a source logic tree.
 *
 * <p>See src/main/resources/application.yml nshmp-haz.model-path for installed
 * model.
 *
 * <p>To run the Micronaut jar file with a model: java -jar
 * path/to/nshmp-haz.jar --model=<path/to/model>
 *
 * @author U.S. Geological Survey
 */
@Tag(
    name = LogicTreesService.NAME,
    description = "USGS NSHM source model logic tree service")
@Controller("${nshmp-haz.context-path}/trees")
public class LogicTreesController {

  @Inject
  private NshmpMicronautServlet servlet;

  @Operation(
      summary = "Source model logic tree listing",
      description = "Returns the IDs of logic trees in the model",
      operationId = "source-tree-metadata")
  @ApiResponse(
      description = "Source logic tree metadata",
      responseCode = "200",
      content = @Content(
          schema = @Schema(
              implementation = MetadataResponse.class)))
  @Get
  public HttpResponse<String> doGetMetadata(HttpRequest<?> http) {
    try {
      return LogicTreesService.getMetadata(http);
    } catch (Exception e) {
      return ServletUtil.error(
          LogicTreesService.LOG, e,
          LogicTreesService.NAME,
          http.getUri().toString());
    }
  }

  /**
   * @param id Source logic tree id
   */
  @Operation(
      summary = "Get a source model feature logic tree",
      description = "Returns the logic tree of features for the supplied ID",
      operationId = "source-tree-features")
  @ApiResponse(
      description = "NSHM source logic tree of features",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = TreeResponse.class)))
  @Get(uri = "/features/{id}")
  public HttpResponse<String> doGetFeatureTree(HttpRequest<?> http, @PathVariable int id) {
    try {
      return LogicTreesService.getFeatureTree(http, id);
    } catch (Exception e) {
      return ServletUtil.error(
          LogicTreesService.LOG, e,
          LogicTreesService.NAME,
          http.getUri().toString());
    }
  }

  /**
   * @param id Source logic tree id
   */
  @Operation(
      summary = "Get a source model MFD logic tree",
      description = "Returns the logic tree of MFDs for the supplied ID",
      operationId = "source-tree-mfds")
  @ApiResponse(
      description = "NSHM source logic tree of MFDs",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = TreeResponse.class)))
  @Get(uri = "/mfds/{id}")
  public HttpResponse<String> doGetMfdTree(HttpRequest<?> http, @PathVariable int id) {
    try {
      return LogicTreesService.getMfdTree(http, id);
    } catch (Exception e) {
      return ServletUtil.error(
          LogicTreesService.LOG, e,
          LogicTreesService.NAME,
          http.getUri().toString());
    }
  }

  // Swagger schema
  private static class MetadataResponse extends ResponseBody<String, Metadata> {}

  // Swagger schema
  private static class TreeResponse extends ResponseBody<RequestData, Object> {}
}
