package gov.usgs.earthquake.nshmp.www.source;

import java.util.Optional;

import gov.usgs.earthquake.nshmp.model.SourceType;
import gov.usgs.earthquake.nshmp.model.TectonicSetting;
import gov.usgs.earthquake.nshmp.www.NshmpMicronautServlet;
import gov.usgs.earthquake.nshmp.www.ResponseBody;
import gov.usgs.earthquake.nshmp.www.ServletUtil;
import gov.usgs.earthquake.nshmp.www.source.MfdsService.Metadata;
import gov.usgs.earthquake.nshmp.www.source.MfdsService.RequestData;

import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.micronaut.http.annotation.PathVariable;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.inject.Inject;

/**
 * Micronaut web service controller for summed MFDs in the installed model.
 * These endpoints return the magnitude frequency distributions (MFDs)
 * associated with the different tectonic settings and source types present in
 * the model. MFDs for requested tetonic settings and source types are scaled by
 * their branch weights and then summed.
 *
 * <p>See src/main/resources/application.yml nshmp-haz.model-path for installed
 * model.
 *
 * <p>To run the Micronaut jar file with a model: java -jar
 * path/to/nshmp-haz.jar --model=<path/to/model>
 *
 * @author U.S. Geological Survey
 */
@Tag(
    name = MfdsService.NAME,
    description = "USGS NSHM source model MFD service")
@Controller("${nshmp-haz.context-path}/mfds")
public class MfdsController {

  @Inject
  private NshmpMicronautServlet servlet;

  @Operation(
      summary = "Source model tectonic settings and source types",
      description = "Returns the tectonic settings and source types in the model",
      operationId = "source-mfd-metadata")
  @ApiResponse(
      description = "Source MFD metadata",
      responseCode = "200",
      content = @Content(
          schema = @Schema(
              implementation = MetadataResponse.class)))
  @Get
  public HttpResponse<String> doGetMetadata(HttpRequest<?> http) {
    try {
      return LogicTreesService.getMetadata(http);
    } catch (Exception e) {
      return ServletUtil.error(
          LogicTreesService.LOG, e,
          LogicTreesService.NAME,
          http.getUri().toString());
    }
  }

  /**
   * @param setting model tectonic setting
   */
  @Operation(
      summary = "Get a source model MFD for a tectonic setting",
      description = "Returns the total MFD for the supplied tectonic setting",
      operationId = "source-setting-mfd")
  @ApiResponse(
      description = "NSHM source MFD",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = Response.class)))
  @Get(uri = "/{setting}")
  public HttpResponse<String> doGetMfdSettingGroup(
      HttpRequest<?> http,
      @PathVariable TectonicSetting setting) {
    try {
      return MfdsService.getMfdGroup(http, setting, Optional.empty());
    } catch (Exception e) {
      return ServletUtil.error(
          LogicTreesService.LOG, e,
          LogicTreesService.NAME,
          http.getUri().toString());
    }
  }

  /**
   * @param setting model tectonic setting
   * @param type model source type}
   */
  @Operation(
      summary = "Get a source model MFD for a tectonic setting and source type",
      description = "Returns the total MFD for the supplied tectonic setting and source type",
      operationId = "source-setting-type-mfd")
  @ApiResponse(
      description = "NSHM source MFD",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = Response.class)))
  @Get(uri = "/{setting}/{type}")
  public HttpResponse<String> doGetMfdTypeGroup(
      HttpRequest<?> http,
      @PathVariable TectonicSetting setting,
      @PathVariable SourceType type) {
    try {
      return MfdsService.getMfdGroup(http, setting, Optional.of(type));
    } catch (Exception e) {
      return ServletUtil.error(
          LogicTreesService.LOG, e,
          LogicTreesService.NAME,
          http.getUri().toString());
    }
  }

  // Swagger schema
  private static class MetadataResponse extends ResponseBody<String, Metadata> {}

  // Swagger schema
  private static class Response extends ResponseBody<RequestData, Object> {}

}
