package gov.usgs.earthquake.nshmp.www.source;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import gov.usgs.earthquake.nshmp.geo.json.FeatureCollection;
import gov.usgs.earthquake.nshmp.www.NshmpMicronautServlet;
import gov.usgs.earthquake.nshmp.www.ResponseBody;
import gov.usgs.earthquake.nshmp.www.ServletUtil;
import gov.usgs.earthquake.nshmp.www.source.FeatureCollectionService.RequestData;

import io.micronaut.core.annotation.Nullable;
import io.micronaut.http.HttpRequest;
import io.micronaut.http.HttpResponse;
import io.micronaut.http.MediaType;
import io.micronaut.http.annotation.Controller;
import io.micronaut.http.annotation.Get;
import io.micronaut.http.annotation.PathVariable;
import io.micronaut.http.annotation.QueryValue;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.inject.Inject;

/**
 * Micronaut web service controller for the GeoJSON test sites associated with
 * the current installed model.
 *
 * <p>See src/main/resources/application.yml nshmp-haz.model-path for installed
 * model.
 *
 * <p>To run the Micronaut jar file with a model: java -jar
 * path/to/nshmp-haz.jar --model=<path/to/model>
 *
 * @author U.S. Geological Survey
 */
@Tag(
    name = TestSitesController.NAME,
    description = "NSHM test sites")
@Controller("${nshmp-haz.context-path}/sites")
public class TestSitesController {
  static final Logger LOG = LoggerFactory.getLogger(TestSitesController.class);
  static final String NAME = "Test Sites";

  @Inject
  private NshmpMicronautServlet servlet;

  @Operation(
      summary = "Get the GeoJSON Feature Collection of test sites",
      description = "Returns the feature collection of test sites",
      operationId = "test-sites")
  @ApiResponse(
      description = "NSHM test sites",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = Response.class)))
  @Get(uri = "{?raw}", produces = MediaType.APPLICATION_JSON)
  public HttpResponse<String> doGet(
      HttpRequest<?> http,
      @QueryValue(defaultValue = "false") @Nullable Boolean raw) {
    try {
      FeatureCollection sites = ServletUtil.model().sites().orElseThrow();
      return HttpResponse
          .ok(FeatureCollectionService.handleFeatureCollection(http, NAME, sites, raw));
    } catch (Exception e) {
      return ServletUtil.error(
          LOG,
          e,
          NAME,
          http.getUri().toString());
    }
  }

  @Operation(
      summary = "Get the GeoJSON Feature Collection of test sites",
      description = "Returns the feature collection of test sites",
      operationId = "test-sites-slash")
  @ApiResponse(
      description = "NSHM test sites",
      responseCode = "200",
      content = @Content(
          schema = @Schema(implementation = Response.class)))
  @Get(uri = "/{raw}", produces = MediaType.APPLICATION_JSON)
  public HttpResponse<String> doGetSlash(
      HttpRequest<?> http,
      @PathVariable(defaultValue = "false") @Nullable Boolean raw) {
    return doGet(http, raw);
  }

  // Swagger schema
  private static class Response extends ResponseBody<RequestData, FeatureCollection> {};
}
